@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 Christian Geier et al.
+# Copyright (c) 2013-2021 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -19,12 +19,22 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
+import datetime as dt
 import logging
 import os
+import stat
 import sys
 import textwrap
 from shutil import get_terminal_size
-import datetime
+
+import click
+import click_log
+
+from . import __version__, controllers, khalendar
+from .exceptions import FatalError
+from .settings import InvalidSettingsError, get_config
+from .settings.exceptions import NoConfigFile
+from .terminal import colored
 
 try:
     from setproctitle import setproctitle
@@ -32,22 +42,13 @@ except ImportError:
     def setproctitle(x):
         pass
 
-import click
 
-from . import controllers, khalendar, __version__
-from .log import logger
-from .settings import get_config, InvalidSettingsError
-from .settings.exceptions import NoConfigFile
-from .exceptions import FatalError
-from .terminal import colored
+logger = logging.getLogger('khal')
+click_log.basic_config('khal')
 
-
-days_option = click.option('--days', default=None, type=int,
-                           help='How many days to include.')
-week_option = click.option('--week', '-w',
-                           help=('Include all events in one week.'), is_flag=True)
-events_option = click.option('--events', default=None, type=int,
-                             help='How many events to include.')
+days_option = click.option('--days', default=None, type=int, help='How many days to include.')
+week_option = click.option('--week', '-w', help='Include all events in one week.', is_flag=True)
+events_option = click.option('--events', default=None, type=int, help='How many events to include.')
 dates_arg = click.argument('dates', nargs=-1)
 
 
@@ -55,43 +56,38 @@ def time_args(f):
     return dates_arg(events_option(week_option(days_option(f))))
 
 
-def _multi_calendar_select_callback(ctx, option, calendars):
-    if not calendars:
-        return
-    if 'calendar_selection' in ctx.obj:
+def multi_calendar_select(ctx, include_calendars, exclude_calendars):
+    if include_calendars and exclude_calendars:
         raise click.UsageError('Can\'t use both -a and -d.')
-    if not isinstance(calendars, tuple):
-        calendars = (calendars,)
+    # if not isinstance(include_calendars, tuple):
+        # include_calendars = (include_calendars,)
+    # if not isinstance(exclude_calendars, tuple):
+        # exclude_calendars = (exclude_calendars,)
 
-    mode = option.name
-    selection = ctx.obj['calendar_selection'] = set()
+    selection = set()
 
-    if mode == 'include_calendar':
-        for cal_name in calendars:
+    if include_calendars:
+        for cal_name in include_calendars:
             if cal_name not in ctx.obj['conf']['calendars']:
                 raise click.BadParameter(
                     'Unknown calendar {}, run `khal printcalendars` to get a '
                     'list of all configured calendars.'.format(cal_name)
                 )
 
-        selection.update(calendars)
-    elif mode == 'exclude_calendar':
+        selection.update(include_calendars)
+    elif exclude_calendars:
         selection.update(ctx.obj['conf']['calendars'].keys())
-        for value in calendars:
+        for value in exclude_calendars:
             selection.remove(value)
-    else:
-        raise ValueError(mode)
+
+    return selection or None
 
 
 def multi_calendar_option(f):
     a = click.option('--include-calendar', '-a', multiple=True, metavar='CAL',
-                     expose_value=False,
-                     callback=_multi_calendar_select_callback,
                      help=('Include the given calendar. Can be specified '
                            'multiple times.'))
     d = click.option('--exclude-calendar', '-d', multiple=True, metavar='CAL',
-                     expose_value=False,
-                     callback=_multi_calendar_select_callback,
                      help=('Exclude the given calendar. Can be specified '
                            'multiple times.'))
 
@@ -118,35 +114,20 @@ def _calendar_select_callback(ctx, option, calendar):
 
 
 def calendar_option(f):
-    return click.option('--calendar', '-a', metavar='CAL',
-                        callback=_calendar_select_callback)(f)
+    return click.option('--calendar', '-a', metavar='CAL', callback=_calendar_select_callback)(f)
 
 
 def global_options(f):
-    def config_callback(ctx, option, config):
-        prepare_context(ctx, config)
-
-    def verbosity_callback(ctx, option, verbose):
-        if verbose:
-            logger.setLevel(logging.DEBUG)
-        else:
-            logger.setLevel(logging.INFO)
-
     def color_callback(ctx, option, value):
         ctx.color = value
 
+    def logfile_callback(ctx, option, path):
+        ctx.logfilepath = path
+
     config = click.option(
         '--config', '-c',
-        is_eager=True,  # make sure other options can access config
         help='The config file to use.',
-        default=None, metavar='PATH', expose_value=False,
-        callback=config_callback
-    )
-    verbose = click.option(
-        '--verbose', '-v',
-        is_eager=True,  # make sure to log config when debugging
-        help='Output debugging information.',
-        is_flag=True, expose_value=False, callback=verbosity_callback
+        default=None, metavar='PATH'
     )
     color = click.option(
         '--color/--no-color',
@@ -156,9 +137,19 @@ def global_options(f):
         callback=color_callback
     )
 
+    logfile = click.option(
+        '--logfile', '-l',
+        help='The logfile to use [defaults to stdout]',
+        type=click.Path(),
+        callback=logfile_callback,
+        default=None,
+        expose_value=False,
+        metavar='LOGFILE',
+    )
+
     version = click.version_option(version=__version__)
 
-    return config(verbose(color(version(f))))
+    return logfile(config(color(version(f))))
 
 
 def build_collection(conf, selection):
@@ -172,6 +163,7 @@ def build_collection(conf, selection):
                     'path': cal['path'],
                     'readonly': cal['readonly'],
                     'color': cal['color'],
+                    'priority': cal['priority'],
                     'ctype': cal['type'],
                 }
         collection = khalendar.CalendarCollection(
@@ -185,6 +177,7 @@ def build_collection(conf, selection):
             highlight_event_days=conf['default']['highlight_event_days'],
         )
     except FatalError as error:
+        logger.debug(error, exc_info=True)
         logger.fatal(error)
         sys.exit(1)
 
@@ -209,7 +202,7 @@ def prepare_context(ctx, config):
     except NoConfigFile:
         conf = _NoConfig()
     except InvalidSettingsError:
-        logger.info('If your configuration file used to work, please have a  '
+        logger.info('If your configuration file used to work, please have a '
                     'look at the Changelog to see what changed.')
         sys.exit(1)
     else:
@@ -236,22 +229,19 @@ def stringify_conf(conf):
 
 
 def _get_cli():
-    @click.group(invoke_without_command=True)
+    @click.group()
+    @click_log.simple_verbosity_option('khal')
     @global_options
     @click.pass_context
-    def cli(ctx):
+    def cli(ctx, config):
         # setting the process title so it looks nicer in ps
         # shows up as 'khal' under linux and as 'python: khal (python2.7)'
         # under FreeBSD, which is still nicer than the default
         setproctitle('khal')
-
-        if not ctx.invoked_subcommand:
-            command = ctx.obj['conf']['default']['default_command']
-            if command:
-                ctx.invoke(cli.commands[command])
-            else:
-                click.echo(ctx.get_help())
-                ctx.exit(1)
+        if ctx.logfilepath:
+            logger = logging.getLogger('khal')
+            logger.handlers = [logging.FileHandler(ctx.logfilepath)]
+        prepare_context(ctx, config)
 
     @cli.command()
     @multi_calendar_option
@@ -267,11 +257,15 @@ def _get_cli():
                   is_flag=True)
     @click.argument('DATERANGE', nargs=-1, required=False)
     @click.pass_context
-    def calendar(ctx, daterange, once, notstarted, format, day_format):
+    def calendar(ctx, include_calendar, exclude_calendar, daterange, once,
+                 notstarted, format, day_format):
         '''Print calendar with agenda.'''
         try:
             rows = controllers.calendar(
-                build_collection(ctx.obj['conf'], ctx.obj.get('calendar_selection', None)),
+                build_collection(
+                    ctx.obj['conf'],
+                    multi_calendar_select(ctx, include_calendar, exclude_calendar)
+                ),
                 agenda_format=format,
                 day_format=day_format,
                 once=once,
@@ -281,6 +275,7 @@ def _get_cli():
                 firstweekday=ctx.obj['conf']['locale']['firstweekday'],
                 locale=ctx.obj['conf']['locale'],
                 weeknumber=ctx.obj['conf']['locale']['weeknumbers'],
+                monthdisplay=ctx.obj['conf']['view']['monthdisplay'],
                 hmethod=ctx.obj['conf']['highlight_days']['method'],
                 default_color=ctx.obj['conf']['highlight_days']['default_color'],
                 multiple=ctx.obj['conf']['highlight_days']['multiple'],
@@ -291,6 +286,7 @@ def _get_cli():
             )
             click.echo('\n'.join(rows))
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
@@ -309,12 +305,16 @@ def _get_cli():
     @click.argument('DATERANGE', nargs=-1, required=False,
                     metavar='[DATETIME [DATETIME | RANGE]]')
     @click.pass_context
-    def klist(ctx, daterange, once, notstarted, format, day_format):
+    def klist(ctx, include_calendar, exclude_calendar,
+              daterange, once, notstarted, format, day_format):
         """List all events between a start (default: today) and (optional)
         end datetime."""
         try:
             event_column = controllers.khal_list(
-                build_collection(ctx.obj['conf'], ctx.obj.get('calendar_selection', None)),
+                build_collection(
+                    ctx.obj['conf'],
+                    multi_calendar_select(ctx, include_calendar, exclude_calendar)
+                ),
                 agenda_format=format,
                 day_format=day_format,
                 daterange=daterange,
@@ -325,6 +325,7 @@ def _get_cli():
             )
             click.echo('\n'.join(event_column))
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
@@ -335,7 +336,7 @@ def _get_cli():
     @click.option('--location', '-l',
                   help=('The location of the new event.'))
     @click.option('--categories', '-g',
-                  help=('The categories of the new event.'))
+                  help=('The categories of the new event, comma separated.'))
     @click.option('--repeat', '-r',
                   help=('Repeat event: daily, weekly, monthly or yearly.'))
     @click.option('--until', '-u',
@@ -344,10 +345,12 @@ def _get_cli():
                   help=('The format to print the event.'))
     @click.option('--alarms', '-m',
                   help=('Alarm times for the new event as DELTAs comma separated'))
+    @click.option('--url', help=("URI for the event."))
     @click.argument('info', metavar='[START [END | DELTA] [TIMEZONE] [SUMMARY] [:: DESCRIPTION]]',
                     nargs=-1)
     @click.pass_context
-    def new(ctx, calendar, info, location, categories, repeat, until, alarms, format, interactive):
+    def new(ctx, calendar, info, location, categories, repeat, until, alarms, url, format,
+            interactive):
         '''Create a new event from arguments.
 
         START and END can be either dates, times or datetimes, please have a
@@ -357,10 +360,9 @@ def _get_cli():
         everything behind them is taken as the event's description.
         '''
         if not info and not interactive:
-                raise click.BadParameter(
-                    'no details provided, '
-                    'did you mean to use --interactive/-i?'
-                )
+            raise click.BadParameter(
+                'no details provided, did you mean to use --interactive/-i?'
+            )
 
         calendar = calendar or ctx.obj['conf']['default']['default_calendar']
         if calendar is None:
@@ -394,9 +396,11 @@ def _get_cli():
                 env={"calendars": ctx.obj['conf']['calendars']},
                 until=until,
                 alarms=alarms,
+                url=url,
                 format=format,
             )
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
@@ -437,7 +441,7 @@ def _get_cli():
             if not ics:
                 ics_strs = (sys.stdin.read(),)
                 if not batch:
-                    if os.path.isfile('/dev/tty'):
+                    if os.stat('/dev/tty').st_mode & stat.S_IFCHR > 0:
                         sys.stdin = open('/dev/tty', 'r')
                     else:
                         logger.warning('/dev/tty does not exist, importing might not work')
@@ -454,16 +458,20 @@ def _get_cli():
                     env={"calendars": ctx.obj['conf']['calendars']},
                 )
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
     @cli.command()
     @multi_calendar_option
     @click.pass_context
-    def interactive(ctx):
+    def interactive(ctx, include_calendar, exclude_calendar):
         '''Interactive UI. Also launchable via `ikhal`.'''
         controllers.interactive(
-            build_collection(ctx.obj['conf'], ctx.obj.get('calendar_selection', None)),
+            build_collection(
+                ctx.obj['conf'],
+                multi_calendar_select(ctx, include_calendar, exclude_calendar)
+            ),
             ctx.obj['conf']
         )
 
@@ -471,24 +479,29 @@ def _get_cli():
     @global_options
     @multi_calendar_option
     @click.pass_context
-    def interactive_cli(ctx):
+    def interactive_cli(ctx, config, include_calendar, exclude_calendar):
         '''Interactive UI. Also launchable via `khal interactive`.'''
+        prepare_context(ctx, config)
         controllers.interactive(
-            build_collection(ctx.obj['conf'], ctx.obj.get('calendar_selection', None)),
-            ctx.obj['conf'])
+            build_collection(
+                ctx.obj['conf'],
+                multi_calendar_select(ctx, include_calendar, exclude_calendar)
+            ),
+            ctx.obj['conf']
+        )
 
     @cli.command()
     @multi_calendar_option
     @click.pass_context
-    def printcalendars(ctx):
+    def printcalendars(ctx, include_calendar, exclude_calendar):
         '''List all calendars.'''
         try:
-            click.echo(
-                '\n'.join(
-                    build_collection(ctx.obj['conf'], ctx.obj.get('calendar_selection', None)).names
-                )
-            )
+            click.echo('\n'.join(build_collection(
+                ctx.obj['conf'],
+                multi_calendar_select(ctx, include_calendar, exclude_calendar)
+            ).names))
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
@@ -497,11 +510,10 @@ def _get_cli():
     def printformats(ctx):
         '''Print a date in all formats.
 
-        Print the date 2013-12-21 10:09 in all configured date(time)
+        Print the date 2013-12-21 21:45 in all configured date(time)
         formats to check if these locale settings are configured to ones
         liking.'''
-        from datetime import datetime
-        time = datetime(2013, 12, 21, 10, 9)
+        time = dt.datetime(2013, 12, 21, 21, 45)
         try:
             for strftime_format in [
                     'longdatetimeformat', 'datetimeformat', 'longdateformat',
@@ -509,6 +521,7 @@ def _get_cli():
                 dt_str = time.strftime(ctx.obj['conf']['locale'][strftime_format])
                 click.echo('{}: {}'.format(strftime_format, dt_str))
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
@@ -530,6 +543,7 @@ def _get_cli():
                 name = 'stdin input'
             controllers.print_ics(ctx.obj['conf'], name, ics_str, format)
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
@@ -539,7 +553,7 @@ def _get_cli():
                   help=('The format of the events.'))
     @click.argument('search_string')
     @click.pass_context
-    def search(ctx, format, search_string):
+    def search(ctx, format, search_string, include_calendar, exclude_calendar):
         '''Search for events matching SEARCH_STRING.
 
         For recurring events, only the master event and different overwritten
@@ -549,11 +563,14 @@ def _get_cli():
         if format is None:
             format = ctx.obj['conf']['view']['event_format']
         try:
-            collection = build_collection(ctx.obj['conf'], ctx.obj.get('calendar_selection', None))
+            collection = build_collection(
+                ctx.obj['conf'],
+                multi_calendar_select(ctx, include_calendar, exclude_calendar)
+            )
             events = sorted(collection.search(search_string))
             event_column = list()
             term_width, _ = get_terminal_size()
-            now = datetime.datetime.now()
+            now = dt.datetime.now()
             env = {"calendars": ctx.obj['conf']['calendars']}
             for event in events:
                 desc = textwrap.wrap(event.format(format, relative_to=now, env=env), term_width)
@@ -564,6 +581,7 @@ def _get_cli():
                 )
             click.echo('\n'.join(event_column))
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
@@ -575,11 +593,14 @@ def _get_cli():
                   is_flag=True)
     @click.argument('search_string', nargs=-1)
     @click.pass_context
-    def edit(ctx, format, search_string, show_past):
+    def edit(ctx, format, search_string, show_past, include_calendar, exclude_calendar):
         '''Interactively edit (or delete) events matching the search string.'''
         try:
             controllers.edit(
-                build_collection(ctx.obj['conf'], ctx.obj.get('calendar_selection', None)),
+                build_collection(
+                    ctx.obj['conf'],
+                    multi_calendar_select(ctx, include_calendar, exclude_calendar)
+                ),
                 ' '.join(search_string),
                 format=format,
                 allow_past=show_past,
@@ -587,6 +608,7 @@ def _get_cli():
                 conf=ctx.obj['conf']
             )
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
@@ -600,13 +622,16 @@ def _get_cli():
                   is_flag=True)
     @click.argument('DATETIME', nargs=-1, required=False, metavar='[[START DATE] TIME | now]')
     @click.pass_context
-    def at(ctx, datetime, notstarted, format, day_format):
+    def at(ctx, datetime, notstarted, format, day_format, include_calendar, exclude_calendar):
         '''Print all events at a specific datetime (defaults to now).'''
         if not datetime:
             datetime = ("now",)
         try:
             rows = controllers.khal_list(
-                build_collection(ctx.obj['conf'], ctx.obj.get('calendar_selection', None)),
+                build_collection(
+                    ctx.obj['conf'],
+                    multi_calendar_select(ctx, include_calendar, exclude_calendar)
+                ),
                 agenda_format=format,
                 day_format=day_format,
                 datepoint=list(datetime),
@@ -617,6 +642,7 @@ def _get_cli():
             )
             click.echo('\n'.join(rows))
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
@@ -628,6 +654,7 @@ def _get_cli():
         try:
             configwizard.configwizard()
         except FatalError as error:
+            logger.debug(error, exc_info=True)
             logger.fatal(error)
             sys.exit(1)
 
