@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 Christian Geier et al.
+# Copyright (c) 2013-2021 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -19,7 +19,9 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from ..exceptions import UnsupportedFeatureError, Error, FatalError
+from typing import Optional  # noqa
+
+from ..exceptions import Error, FatalError, UnsupportedFeatureError
 
 
 class UnsupportedRruleExceptionError(UnsupportedFeatureError):
@@ -54,13 +56,12 @@ class UpdateFailed(Error):
     """could not update the event in the database"""
 
 
-class UnsupportedRecurrence(Error):
-
-    """raised if the RRULE is not understood by dateutil.rrule"""
-    pass
-
-
 class DuplicateUid(Error):
 
     """an event with this UID already exists"""
-    existing_href = None
+    existing_href = None  # type: Optional[str]
+
+
+class NonUniqueUID(Error):
+
+    """the .ics file contains more than one UID"""
