@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2016 Christian Geier et al.
+# Copyright (c) 2013-2021 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,58 +20,66 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
+import logging
 import os
 
-from configobj import ConfigObj, flatten_errors, get_extra_values, \
-    ConfigObjError
-from validate import Validator
 import xdg.BaseDirectory
-
-from .exceptions import InvalidSettingsError, CannotParseConfigFileError, NoConfigFile
+from configobj import (ConfigObj, ConfigObjError, flatten_errors,
+                       get_extra_values)
 from khal import __productname__
-from ..log import logger
-from .utils import is_timezone, weeknumber_option, config_checks, \
-    expand_path, expand_db_path, is_color
+from validate import Validator
 
+from .exceptions import (CannotParseConfigFileError, InvalidSettingsError,
+                         NoConfigFile)
+from .utils import (config_checks, expand_db_path, expand_path,
+                    get_color_from_vdir, get_vdir_type, is_color, is_timedelta,
+                    is_timezone, weeknumber_option, monthdisplay_option)
+
+logger = logging.getLogger('khal')
 SPECPATH = os.path.join(os.path.dirname(__file__), 'khal.spec')
 
 
 def find_configuration_file():
     """Return the configuration filename.
 
-    This function builds the list of paths known by khal and
-    then return the first one which exists. The first paths
-    searched are the ones described in the XDG Base Directory
-    Standard. Each one of this path ends with
-    DEFAULT_PATH/DEFAULT_FILE.
-
-    On failure, the path DEFAULT_PATH/DEFAULT_FILE, prefixed with
-    a dot, is searched in the home user directory. Ultimately,
-    DEFAULT_FILE is searched in the current directory.
+    This function builds the list of paths known by khal and then return the
+    first one which exists. The first paths searched are the ones described in
+    the XDG Base Directory Standard, e.g. ~/.config/khal/config, additionally
+    ~/.config/khal/khal.conf is searched (deprecated).
     """
-    DEFAULT_FILE = __productname__ + '.conf'
     DEFAULT_PATH = __productname__
-    resource = os.path.join(DEFAULT_PATH, DEFAULT_FILE)
 
     paths = []
-    paths.extend([os.path.join(path, resource)
-                  for path in xdg.BaseDirectory.xdg_config_dirs])
-    paths.append(os.path.expanduser(os.path.join('~', '.' + resource)))
-    paths.append(os.path.expanduser(DEFAULT_FILE))
-
+    paths = [os.path.join(path, os.path.join(DEFAULT_PATH, 'config'))
+             for path in xdg.BaseDirectory.xdg_config_dirs]
     for path in paths:
         if os.path.exists(path):
             return path
 
+    # remove this part for v0.11.0
+    for path in paths:
+        if os.path.exists(path):
+            logger.warning(
+                'Deprecation Warning: configuration file path `{}` will not be '
+                'supported from v0.11.0 onwards, please move it to '
+                '`{}/khal/config`.'
+                ''.format(path, xdg.BaseDirectory.xdg_config_dirs[0]))
+            return path
+
     return None
 
 
-def get_config(config_path=None):
+def get_config(
+        config_path=None,
+        _get_color_from_vdir=get_color_from_vdir,
+        _get_vdir_type=get_vdir_type):
     """reads the config file, validates it and return a config dict
 
     :param config_path: path to a custom config file, if none is given the
                         default locations will be searched
     :type config_path: str
+    :param _get_color_from_vdir: override get_color_from_vdir for testing purposes
+    :param _get_vdir_type: override get_vdir_type for testing purposes
     :returns: configuration
     :rtype: dict
     """
@@ -89,7 +97,7 @@ def get_config(config_path=None):
                                 file_error=True,
                                 )
     except ConfigObjError as error:
-        logger.fatal('parsing the config file file with the following error: '
+        logger.fatal('parsing the config file with the following error: '
                      '{}'.format(error))
         logger.fatal('if you recently updated khal, the config file format '
                      'might have changed, in that case please consult the '
@@ -97,43 +105,51 @@ def get_config(config_path=None):
         raise CannotParseConfigFileError()
 
     fdict = {'timezone': is_timezone,
+             'timedelta': is_timedelta,
              'expand_path': expand_path,
              'expand_db_path': expand_db_path,
              'weeknumbers': weeknumber_option,
+             'monthdisplay': monthdisplay_option,
              'color': is_color,
              }
     validator = Validator(fdict)
     results = user_config.validate(validator, preserve_errors=True)
 
     abort = False
-    for section, subsection, error in flatten_errors(user_config, results):
+    for section, subsection, config_error in flatten_errors(user_config, results):
         abort = True
-        if isinstance(error, Exception):
+        if isinstance(config_error, Exception):
             logger.fatal(
                 'config error:\n'
-                'in [{}] {}: {}'.format(section[0], subsection, error))
+                'in [{}] {}: {}'.format(section[0], subsection, config_error))
         else:
-            for key in error:
-                if isinstance(error[key], Exception):
+            for key in config_error:
+                if isinstance(config_error[key], Exception):
                     logger.fatal('config error:\nin {} {}: {}'.format(
                         sectionize(section + [subsection]),
                         key,
-                        str(error[key]))
+                        str(config_error[key]))
                     )
 
     if abort or not results:
         raise InvalidSettingsError()
 
-    config_checks(user_config)
+    config_checks(user_config, _get_color_from_vdir, _get_vdir_type)
 
     extras = get_extra_values(user_config)
     for section, value in extras:
         if section == ():
-            logger.warn('unknown section "{}" in config file'.format(value))
+            logger.warning('unknown section "{}" in config file'.format(value))
         else:
             section = sectionize(section)
-            logger.warn('unknown key or subsection "{}" in '
-                        'section "{}"'.format(value, section))
+            logger.warning(
+                'unknown key or subsection "{}" in section "{}"'.format(value, section))
+
+            deprecated = [{'value': 'default_command', 'section': 'default'}]
+            for d in deprecated:
+                if (value == d['value']) and (section == d['section']):
+                    logger.warning('Key "{}" in section "{}" was deprecated. '
+                                   'See the FAQ to find out when and why!'.format(value, section))
     return user_config
 
 
