@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2016 Christian Geier et al.
+# Copyright (c) 2013-2021 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,20 +20,22 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-from os.path import expandvars, expanduser, join
+import glob
+import logging
 import os
+from os.path import expanduser, expandvars, join
 
 import pytz
 import xdg
 from tzlocal import get_localzone
 from validate import VdtValueError
 
-from ..log import logger
+from ..khalendar.vdir import CollectionNotFoundError, Vdir
+from ..parse_datetime import guesstimedeltafstr
+from ..terminal import COLORS
 from .exceptions import InvalidSettingsError
 
-from ..terminal import COLORS
-from vdirsyncer.storage.filesystem import FilesystemStorage
-from vdirsyncer.exceptions import CollectionNotFound
+logger = logging.getLogger('khal')
 
 
 def is_timezone(tzstring):
@@ -49,6 +51,13 @@ def is_timezone(tzstring):
         raise VdtValueError("Unknown timezone {}".format(tzstring))
 
 
+def is_timedelta(string):
+    try:
+        return guesstimedeltafstr(string)
+    except ValueError:
+        raise VdtValueError("Invalid timedelta: {}".format(string))
+
+
 def weeknumber_option(option):
     """checks if *option* is a valid value
 
@@ -70,6 +79,25 @@ def weeknumber_option(option):
             "'off', 'left' or 'right'".format(option))
 
 
+def monthdisplay_option(option):
+    """checks if *option* is a valid value
+
+    :param option: the option the user set in the config file
+    :type option: str
+    :returns: firstday, firstfullweek
+    :rtype: str/bool
+    """
+    option = option.lower()
+    if option == 'firstday':
+        return 'firstday'
+    elif option == 'firstfullweek':
+        return 'firstfullweek'
+    else:
+        raise VdtValueError(
+            "Invalid value '{}' for option 'monthdisplay', must be one of "
+            "'firstday' or 'firstfullweek'".format(option))
+
+
 def expand_path(path):
     """expands `~` as well as variable names"""
     return expanduser(expandvars(path))
@@ -92,7 +120,7 @@ def is_color(color):
     # 2) auto
     # 3) a color name from the 16 color palette
     # 4) a color index from the 256 color palette
-    # 5) a HTML-style color code
+    # 5) an HTML-style color code
     if (color in ['', 'auto'] or
             color in COLORS.keys() or
             (color.isdigit() and int(color) >= 0 and int(color) <= 255) or
@@ -115,22 +143,35 @@ def test_default_calendar(config):
                                        config['calendars'].keys())
         )
         raise InvalidSettingsError()
+    elif config['calendars'][config['default']['default_calendar']]['readonly']:
+        logger.fatal('default_calendar may not be read_only!')
+        raise InvalidSettingsError()
 
 
 def get_color_from_vdir(path):
     try:
-        color = FilesystemStorage(path, '.ics').get_meta('color')
-    except CollectionNotFound:
+        color = Vdir(path, '.ics').get_meta('color')
+    except CollectionNotFoundError:
         color = None
-    if color is None or color is '':
+    if color is None or color == '':
         logger.debug('Found no or empty file `color` in {}'.format(path))
+        return None
+    color = color.strip()
+    try:
+        is_color(color)
+    except VdtValueError:
+        logger.warning("Found invalid color `{}` in {}color".format(color, path))
         color = None
     return color
 
 
 def get_unique_name(path, names):
     # TODO take care of edge cases, make unique name finding less brain-dead
-    name = FilesystemStorage(path, '.ics').get_meta('displayname')
+    try:
+        name = Vdir(path, '.ics').get_meta('displayname')
+    except CollectionNotFoundError:
+        logger.fatal('The calendar at `{}` is not a directory.'.format(path))
+        raise
     if name is None or name == '':
         logger.debug('Found no or empty file `displayname` in {}'.format(path))
         name = os.path.split(path)[-1]
@@ -141,21 +182,10 @@ def get_unique_name(path, names):
 
 
 def get_all_vdirs(path):
-    """returns (recursively) all directories under `path` that contain
-    only files (not directories).
+    """returns a list of paths, expanded using glob
     """
-    # TODO take care of links
-    vdirs = list()
-    contains_only_file = True
-    items = os.listdir(path)
-    for item in items:
-        itempath = os.path.join(path, item)
-        if os.path.isdir(itempath):
-            contains_only_file = False
-            vdirs += get_all_vdirs(itempath)
-    if contains_only_file:
-        vdirs.append(path)
-    return vdirs
+    items = glob.glob(path)
+    return items
 
 
 def get_vdir_type(_):
@@ -163,7 +193,10 @@ def get_vdir_type(_):
     return 'calendar'
 
 
-def config_checks(config):
+def config_checks(
+        config,
+        _get_color_from_vdir=get_color_from_vdir,
+        _get_vdir_type=get_vdir_type):
     """do some tests on the config we cannot do with configobj's validator"""
     if len(config['calendars'].keys()) < 1:
         logger.fatal('Found no calendar section in the config file')
@@ -177,17 +210,36 @@ def config_checks(config):
             config['locale']['local_timezone'])
 
     # expand calendars with type = discover
-    vdirs = list()
+    vdirs_complete = list()
+    vdir_colors_from_config = {}
     for calendar in list(config['calendars'].keys()):
+        if not isinstance(config['calendars'][calendar], dict):
+            logger.fatal('Invalid config file, probably missing calendar sections')
+            raise InvalidSettingsError
         if config['calendars'][calendar]['type'] == 'discover':
-            vdirs += get_all_vdirs(config['calendars'][calendar]['path'])
+            logger.debug(
+                'discovering calendars in {}'.format(config['calendars'][calendar]['path'])
+            )
+            vdirs = get_all_vdirs(config['calendars'][calendar]['path'])
+            vdirs_complete += vdirs
+            if 'color' in config['calendars'][calendar]:
+                for vdir in vdirs:
+                    vdir_colors_from_config[vdir] = config['calendars'][calendar]['color']
             config['calendars'].pop(calendar)
-    for vdir in sorted(vdirs):
+    for vdir in sorted(vdirs_complete):
         calendar = {'path': vdir,
-                    'color': get_color_from_vdir(vdir),
-                    'type': get_vdir_type(vdir),
-                    'readonly': False
+                    'color': _get_color_from_vdir(vdir),
+                    'type': _get_vdir_type(vdir),
+                    'readonly': False,
+                    'priority': 10,
                     }
+
+        # get color from config if not defined in vdir
+
+        if calendar['color'] is None and vdir in vdir_colors_from_config:
+            logger.debug("using collection's color for {}".format(vdir))
+            calendar['color'] = vdir_colors_from_config[vdir]
+
         name = get_unique_name(vdir, config['calendars'].keys())
         config['calendars'][name] = calendar
 
@@ -197,4 +249,4 @@ def config_checks(config):
             config['calendars'][calendar]['readonly'] = True
         if config['calendars'][calendar]['color'] == 'auto':
             config['calendars'][calendar]['color'] = \
-                get_color_from_vdir(config['calendars'][calendar]['path'])
+                _get_color_from_vdir(config['calendars'][calendar]['path'])
