@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2016 Christian Geier et al.
+# Copyright (c) 2013-2021 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -19,13 +19,18 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-"""contains a re-usable CalendarWidget for urwid"""
+"""Contains a re-usable CalendarWidget for urwid.
+
+if anything doesn't work as expected, please open an issue for khal
+"""
 
 import calendar
-from datetime import date
-from locale import getlocale, setlocale, LC_ALL
+import datetime as dt
+from collections import defaultdict
+from locale import LC_ALL, LC_TIME, getlocale, setlocale
 
 import urwid
+from khal.utils import get_month_abbr_len
 
 setlocale(LC_ALL, '')
 
@@ -37,7 +42,7 @@ def getweeknumber(day):
     :return: weeknumber
     :rtype: int
     """
-    return date.isocalendar(day)[1]
+    return dt.date.isocalendar(day)[1]
 
 
 class DatePart(urwid.Text):
@@ -54,6 +59,16 @@ class DatePart(urwid.Text):
     def keypress(self, _, key):
         return key
 
+    def get_cursor_coords(self, size):
+        return 1, 0
+
+    def render(self, size, focus=False):
+        canv = super().render(size, focus)
+        if focus:
+            canv = urwid.CompositeCanvas(canv)
+            canv.cursor = 1, 0
+        return canv
+
 
 class Date(urwid.WidgetWrap):
 
@@ -79,8 +94,8 @@ class Date(urwid.WidgetWrap):
         else:
             self.halves[0].set_attr_map({None: styles})
             self.halves[1].set_attr_map({None: styles})
-            self.halves[1].set_focus_map({None: styles})
             self.halves[0].set_focus_map({None: styles})
+            self.halves[1].set_focus_map({None: styles})
 
     def reset_styles(self, focus=False):
         self.set_styles(self._get_styles(self.date, focus))
@@ -110,6 +125,8 @@ class DateCColumns(urwid.Columns):
     focus can only move away by pressing 'TAB',
     calls 'on_date_change' on every focus change (see below for details)
     """
+    # TODO only call on_date_change when we change our date ourselves,
+    # not if it gets changed by an (external) call to set_focus_date()
 
     def __init__(self, widget_list, on_date_change, on_press, keybindings,
                  get_styles=None, **kwargs):
@@ -117,10 +134,6 @@ class DateCColumns(urwid.Columns):
         self.on_press = on_press
         self.keybindings = keybindings
         self.get_styles = get_styles
-        # we need the next two attributes for attribute resetting when a
-        # cell regains focus after having lost it
-        self._old_attr_map = False
-        self._old_pos = 0
         self._init = True
         super(DateCColumns, self).__init__(widget_list, **kwargs)
 
@@ -222,9 +235,9 @@ class CListBox(urwid.ListBox):
         super(CListBox, self).__init__(walker)
 
     def render(self, size, focus=False):
+        while 'bottom' in self.ends_visible(size):
+            self.body._autoextend()
         if self._init:
-            while 'bottom' in self.ends_visible(size):
-                self.body._autoextend()
             self.set_focus_valign('middle')
             self._init = False
 
@@ -291,6 +304,7 @@ class CListBox(urwid.ListBox):
                 self._unmark_one(row, col)
 
     def set_focus_date(self, a_day):
+        self.focus.focus.set_styles(self.focus.get_styles(self.body.focus_date, False))
         if self._marked:
             self._unmark_all()
             self._mark(a_day)
@@ -298,9 +312,9 @@ class CListBox(urwid.ListBox):
 
     def keypress(self, size, key):
         if key in self.keybindings['mark'] + ['esc'] and self._marked:
-                self._unmark_all()
-                self._marked = False
-                return
+            self._unmark_all()
+            self._marked = False
+            return
         if key in self.keybindings['mark']:
             self._marked = {'date': self.body.focus_date,
                             'pos': (self.focus_position, self.focus.focus_col)}
@@ -321,10 +335,9 @@ class CListBox(urwid.ListBox):
             return self.on_press[key](start, end)
         if key in self.keybindings['today'] + ['page down', 'page up']:
             # reset colors of currently focused Date widget
-            self.focus.focus.set_styles(
-                self.focus.get_styles(self.body.focus_date, False))
+            self.focus.focus.set_styles(self.focus.get_styles(self.body.focus_date, False))
         if key in self.keybindings['today']:
-            self.set_focus_date(date.today())
+            self.set_focus_date(dt.date.today())
             self.set_focus_valign(('relative', 10))
 
         key = super(CListBox, self).keypress(size, key)
@@ -335,11 +348,13 @@ class CListBox(urwid.ListBox):
 
 class CalendarWalker(urwid.SimpleFocusListWalker):
     def __init__(self, on_date_change, on_press, keybindings, firstweekday=0,
-                 weeknumbers=False, get_styles=None, initial=None):
+                 weeknumbers=False, monthdisplay='firstday', get_styles=None,
+                 initial=None):
         if initial is None:
-            initial = date.today()
+            initial = dt.date.today()
         self.firstweekday = firstweekday
         self.weeknumbers = weeknumbers
+        self.monthdisplay = monthdisplay
         self.on_date_change = on_date_change
         self.on_press = on_press
         self.keybindings = keybindings
@@ -373,10 +388,20 @@ class CalendarWalker(urwid.SimpleFocusListWalker):
         self.set_focus(row)
         self[self.focus]._set_focus_position(column)
 
+    @property
+    def earliest_date(self):
+        """return earliest day that is already loaded into the CalendarWidget"""
+        return self[0][1].date
+
+    @property
+    def latest_date(self):
+        """return latest day that is already loaded into the CalendarWidget"""
+        return self[-1][7].date
+
     def reset_styles_range(self, min_date, max_date):
-        """reset styles for all dates between min_date and max_date"""
-        minr, minc = self.get_date_pos(min_date)
-        maxr, maxc = self.get_date_pos(max_date)
+        """reset styles for all (displayed) dates between min_date and max_date"""
+        minr, minc = self.get_date_pos(max(min_date, self.earliest_date))
+        maxr, maxc = self.get_date_pos(min(max_date, self.latest_date))
         focus_pos = self.focus, self[self.focus].focus_col
 
         for row in range(minr, maxr + 1):
@@ -460,7 +485,10 @@ class CalendarWalker(urwid.SimpleFocusListWalker):
                   if today is in this week
         :rtype: tuple(urwid.CColumns, bool)
         """
-        if 1 in [day.day for day in week]:
+        if self.monthdisplay == 'firstday' and 1 in [day.day for day in week]:
+            month_name = calendar.month_abbr[week[-1].month].ljust(4)
+            attr = 'monthname'
+        elif self.monthdisplay == 'firstfullweek' and week[0].day <= 7:
             month_name = calendar.month_abbr[week[-1].month].ljust(4)
             attr = 'monthname'
         elif self.weeknumbers == 'left':
@@ -470,7 +498,7 @@ class CalendarWalker(urwid.SimpleFocusListWalker):
             month_name = '    '
             attr = None
 
-        this_week = [(4, urwid.AttrMap(urwid.Text(month_name), attr))]
+        this_week = [(get_month_abbr_len(), urwid.AttrMap(urwid.Text(month_name), attr))]
         for number, day in enumerate(week):
             new_date = Date(day, self.get_styles)
             this_week.append((2, new_date))
@@ -488,8 +516,8 @@ class CalendarWalker(urwid.SimpleFocusListWalker):
         return week
 
     def _construct_month(self,
-                         year=date.today().year,
-                         month=date.today().month,
+                         year=dt.date.today().year,
+                         month=dt.date.today().month,
                          clean_first_row=False,
                          clean_last_row=False):
         """construct one month of DateCColumns
@@ -530,33 +558,35 @@ class CalendarWalker(urwid.SimpleFocusListWalker):
 
 class CalendarWidget(urwid.WidgetWrap):
     def __init__(self, on_date_change, keybindings, on_press, firstweekday=0,
-                 weeknumbers=False, get_styles=None, initial=None):
-
+                 weeknumbers=False, monthdisplay='firstday', get_styles=None, initial=None):
         """
-        :param on_date_change: a function that is called every time the selected date
-                        is changed with the newly selected date as a first (and
-                        only argument)
+        :param on_date_change: a function that is called every time the selected
+            date is changed with the newly selected date as a first (and only
+            argument)
         :type on_date_change: function
-        :param keybindings: bind keys to specific functions, keys are
-            commands (e.g. movement commands, values are lists of keys
-            that should be bound to those commands. See below for the
-            defaults.
+        :param keybindings: bind keys to specific functionionality, keys are
+            the available commands, values are lists of keys that should be
+            bound to those commands. See below for the defaults.
             Available commands:
                 'left', 'right', 'up', 'down': move cursor in direction
                 'today': refocus on today
                 'mark': toggles selection mode
         :type keybindings: dict
-        :param on_press: dict of functions that are called when the key is
-            pressed. These functions must accept at least two argument. In the
-            normal case the first argument is the currently selected date
-            (datetime.date) and the second is *None*. When a date range is
-            selected, the first argument is the earlier and the second argument
+        :param on_press: dictonary of functions that are called when the key is
+            pressed and is not already bound to one of the internal functionions
+            via `keybindings`. These functions must accept two arguments, in
+            normal mode the first argument is the currently selected date
+            (datetime.date) and the second is `None`. When a date range is
+            selected, the first argument is the earlier, the second argument
             is the later date. The function's return values are interpreted as
-            pressed keys.
-        :type on_pres: dict
+            pressed keys, which are handed to the widget containing the
+            CalendarWidget.
+        :type on_press: dict
         """
         if initial is None:
-            self._initial = date.today()
+            self._initial = dt.date.today()
+        else:
+            self._initial = initial
 
         default_keybindings = {
             'left': ['left'], 'down': ['down'], 'right': ['right'], 'up': ['up'],
@@ -564,14 +594,12 @@ class CalendarWidget(urwid.WidgetWrap):
             'view': [],
             'mark': ['v'],
         }
-        from collections import defaultdict
         on_press = defaultdict(lambda: lambda x: x, on_press)
-
         default_keybindings.update(keybindings)
         calendar.setfirstweekday(firstweekday)
 
         try:
-            mylocale = '.'.join(getlocale())
+            mylocale = '.'.join(getlocale(LC_TIME))
         except TypeError:  # language code and encoding may be None
             mylocale = 'C'
 
@@ -581,12 +609,12 @@ class CalendarWidget(urwid.WidgetWrap):
 
         def _get_styles(date, focus):
             if focus:
-                if date == date.today():
+                if date == dt.date.today():
                     return 'today focus'
                 else:
                     return 'reveal focus'
             else:
-                if date == date.today():
+                if date == dt.date.today():
                     return 'today'
                 else:
                     return None
@@ -595,12 +623,14 @@ class CalendarWidget(urwid.WidgetWrap):
 
         if weeknumbers == 'right':
             dnames.append('#w')
+        month_names_length = get_month_abbr_len()
         dnames = urwid.Columns(
-            [(4, urwid.Text('    '))] +
+            [(month_names_length, urwid.Text(' ' * month_names_length))] +
             [(2, urwid.AttrMap(urwid.Text(name), 'dayname')) for name in dnames],
             dividechars=1)
         self.walker = CalendarWalker(
-            on_date_change, on_press, default_keybindings, firstweekday, weeknumbers,
+            on_date_change, on_press, default_keybindings, firstweekday,
+            weeknumbers, monthdisplay,
             get_styles, initial=self._initial)
         self.box = CListBox(self.walker)
         frame = urwid.Frame(self.box, header=dnames)
@@ -608,11 +638,15 @@ class CalendarWidget(urwid.WidgetWrap):
         self.set_focus_date(self._initial)
 
     def focus_today(self):
-        self.set_focus_date(date.today())
+        self.set_focus_date(dt.date.today())
 
     def reset_styles_range(self, min_date, max_date):
         self.walker.reset_styles_range(min_date, max_date)
 
+    @classmethod
+    def selectable(cls):
+        return True
+
     @property
     def focus_date(self):
         return self.walker.focus_date
