@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 Christian Geier et al.
+# Copyright (c) 2013-2021 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -36,9 +36,9 @@ dark = [
     ('today focus', 'white', 'dark magenta'),
     ('today', 'dark gray', 'dark green',),
 
-    ('date', 'light gray', ''),
-    ('date focused', 'black', 'light gray', ('bold', 'standout')),
-    ('date selected', 'white', 'yellow'),
+    ('date header', 'light gray', 'black'),
+    ('date header focused', 'black', 'white'),
+    ('date header selected', 'dark gray', 'light gray'),
 
     ('dayname', 'light gray', ''),
     ('monthname', 'light gray', ''),
@@ -51,7 +51,6 @@ dark = [
     ('frame focus color', 'dark blue', 'black'),
     ('frame focus top', 'dark magenta', 'black'),
 
-    ('editfc', 'white', 'dark blue', 'bold'),
     ('editbx', 'light gray', 'dark blue'),
     ('editcp', 'black', 'light gray', 'standout'),
     ('popupbg', 'white', 'black', 'bold'),
@@ -72,9 +71,9 @@ light = [
     ('today focus', 'white', 'dark cyan', 'standout'),
     ('today', 'black', 'light gray', 'dark cyan'),
 
-    ('date', '', 'white'),
-    ('date focused', 'white', 'dark gray', ('bold', 'standout')),
-    ('date selected', 'dark gray', 'light cyan'),
+    ('date header', '', 'white'),
+    ('date header focused', 'white', 'dark gray', ('bold', 'standout')),
+    ('date header selected', 'dark gray', 'light cyan'),
 
     ('dayname', 'dark gray', 'white'),
     ('monthname', 'dark gray', 'white'),
@@ -87,7 +86,6 @@ light = [
     ('frame focus color', 'dark blue', 'white'),
     ('frame focus top', 'dark magenta', 'white'),
 
-    ('editfc', 'white', 'dark blue', 'bold'),
     ('editbx', 'light gray', 'dark blue'),
     ('editcp', 'black', 'light gray', 'standout'),
     ('popupbg', 'white', 'black', 'bold'),
