@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2016 Christian Geier et al.
+# Copyright (c) 2013-2021 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -24,7 +24,7 @@
 Widgets that are specific to calendaring/khal should go into __init__.py or,
 if they are large, into their own files
 """
-from datetime import date, datetime, timedelta
+import datetime as dt
 import re
 
 import urwid
@@ -171,12 +171,12 @@ class DateTimeWidget(ExtendedEdit):
 
 
 class DateWidget(DateTimeWidget):
-    dtype = date
-    timedelta = timedelta(days=1)
+    dtype = dt.date
+    timedelta = dt.timedelta(days=1)
 
     def _get_current_value(self):
         try:
-            new_date = datetime.strptime(self.get_edit_text(), self.dateformat).date()
+            new_date = dt.datetime.strptime(self.get_edit_text(), self.dateformat).date()
         except ValueError:
             raise DateConversionError
         else:
@@ -184,12 +184,12 @@ class DateWidget(DateTimeWidget):
 
 
 class TimeWidget(DateTimeWidget):
-    dtype = datetime
-    timedelta = timedelta(minutes=15)
+    dtype = dt.datetime
+    timedelta = dt.timedelta(minutes=15)
 
     def _get_current_value(self):
         try:
-            new_datetime = datetime.strptime(self.get_edit_text(), self.dateformat)
+            new_datetime = dt.datetime.strptime(self.get_edit_text(), self.dateformat)
         except ValueError:
             raise DateConversionError
         else:
@@ -197,16 +197,20 @@ class TimeWidget(DateTimeWidget):
 
 
 class Choice(urwid.PopUpLauncher):
-    def __init__(self, choices, active, decorate_func=None, overlay_width=32):
+    def __init__(
+            self, choices, active, decorate_func=None, overlay_width=32, callback=lambda: None,
+    ):
         self.choices = choices
+        self._callback = callback
         self._decorate = decorate_func or (lambda x: x)
         self._overlay_width = overlay_width
         self.active = self._original = active
 
     def create_pop_up(self):
-        pop_up = ChoiceList(self)
-        urwid.connect_signal(pop_up, 'close',
-                             lambda button: self.close_pop_up())
+        pop_up = ChoiceList(self, callback=self._callback)
+        urwid.connect_signal(
+            pop_up, 'close', lambda button: self.close_pop_up(),
+        )
         return pop_up
 
     def get_pop_up_parameters(self):
@@ -233,10 +237,12 @@ class Choice(urwid.PopUpLauncher):
 
 
 class ChoiceList(urwid.WidgetWrap):
+    """A pile of Button() widgets, intended to be used with Choice()"""
     signals = ['close']
 
-    def __init__(self, parent):
+    def __init__(self, parent, callback=lambda: None):
         self.parent = parent
+        self._callback = callback
         buttons = []
         for c in parent.choices:
             buttons.append(
@@ -245,11 +251,14 @@ class ChoiceList(urwid.WidgetWrap):
             )
 
         pile = NPile(buttons, outermost=True)
+        num = [num for num, elem in enumerate(parent.choices) if elem == parent.active][0]
+        pile.set_focus(num)
         fill = urwid.Filler(pile)
         urwid.WidgetWrap.__init__(self, urwid.AttrMap(fill, 'popupbg'))
 
     def set_choice(self, button, account):
         self.parent.active = account
+        self._callback()
         self._emit("close")
 
 
@@ -383,7 +392,7 @@ class ValidatedEdit(urwid.WidgetWrap):
     def __init__(self, *args, EditWidget=ExtendedEdit, validate=False, **kwargs):
         assert validate
         self._validate_func = validate
-        self._original_widget = urwid.AttrMap(EditWidget(*args, **kwargs), 'edit', 'editf')
+        self._original_widget = urwid.AttrMap(EditWidget(*args, **kwargs), 'edit', 'edit focused')
         super().__init__(self._original_widget)
 
     @property
@@ -427,6 +436,20 @@ class ValidatedEdit(urwid.WidgetWrap):
         return super().keypress(size, key)
 
 
+class PositiveIntEdit(ValidatedEdit):
+    def __init__(self, *args, EditWidget=ExtendedEdit, validate=False, **kwargs):
+        """Variant of Validated Edit that only accepts positive integers."""
+        super().__init__(*args, validate=self._unsigned_int, **kwargs)
+
+    @staticmethod
+    def _unsigned_int(number):
+        """test if `number` can be converted to a positive int"""
+        try:
+            return int(number) >= 0
+        except ValueError:
+            return False
+
+
 class DurationWidget(urwid.WidgetWrap):
 
     @staticmethod
@@ -472,7 +495,7 @@ class DurationWidget(urwid.WidgetWrap):
         urwid.WidgetWrap.__init__(self, self.columns)
 
     def get_timedelta(self):
-        return timedelta(
+        return dt.timedelta(
             seconds=int(self.seconds_edit.get_edit_text()) +
             int(self.minutes_edit.get_edit_text()) * 60 +
             int(self.hours_edit.get_edit_text()) * 60 * 60 +
@@ -492,7 +515,8 @@ class AlarmsEditor(urwid.WidgetWrap):
                 duration = -1 * duration
 
             self.duration = DurationWidget(duration)
-            self.description = ExtendedEdit(edit_text=description)
+            self.description = ExtendedEdit(
+                edit_text=description if description is not None else "")
             self.direction = Choice(
                 ['before', 'after'], active=direction, overlay_width=10)
             self.columns = NColumns([
@@ -526,7 +550,7 @@ class AlarmsEditor(urwid.WidgetWrap):
     def add_alarm(self, button):
         self.pile.contents.insert(
             len(self.pile.contents) - 1,
-            (self.AlarmEditor((timedelta(0), self.event.summary), self.remove_alarm),
+            (self.AlarmEditor((dt.timedelta(0), self.event.summary), self.remove_alarm),
              ('weight', 1)))
 
     def remove_alarm(self, button, editor):
@@ -545,3 +569,122 @@ class AlarmsEditor(urwid.WidgetWrap):
             return self.event.alarms != self.get_alarms()
         except ValueError:
             return False
+
+
+class FocusLineBoxWidth(urwid.WidgetDecoration, urwid.WidgetWrap):
+    def __init__(self, widget):
+        hline = urwid.Divider('─')
+        hline_focus = urwid.Divider('━')
+        self._vline = urwid.SolidFill('│')
+        self._vline_focus = urwid.SolidFill('┃')
+        self._topline = urwid.Columns([
+            ('fixed', 1, urwid.Text('┌')),
+            hline,
+            ('fixed', 1, urwid.Text('┐')),
+        ])
+        self._topline_focus = urwid.Columns([
+            ('fixed', 1, urwid.Text('┏')),
+            hline_focus,
+            ('fixed', 1, urwid.Text('┓')),
+        ])
+        self._bottomline = urwid.Columns([
+            ('fixed', 1, urwid.Text('└')),
+            hline,
+            ('fixed', 1, urwid.Text('┘')),
+        ])
+        self._bottomline_focus = urwid.Columns([
+            ('fixed', 1, urwid.Text('┗')),
+            hline_focus,
+            ('fixed', 1, urwid.Text('┛')),
+        ])
+        self._middle = urwid.Columns(
+            [('fixed', 1, self._vline), widget, ('fixed', 1, self._vline)],
+            focus_column=1,
+        )
+        self._all = urwid.Pile(
+            [('flow', self._topline), self._middle, ('flow', self._bottomline)],
+            focus_item=1,
+        )
+
+        urwid.WidgetDecoration.__init__(self, widget)
+        urwid.WidgetWrap.__init__(self, self._all)
+
+    def render(self, size, focus):
+        inner = self._all.contents[1][0]
+        if focus:
+            self._all.contents[0] = (self._topline_focus, ('pack', None))
+            inner.contents[0] = (self._vline_focus, ('given', 1, False))
+            inner.contents[2] = (self._vline_focus, ('given', 1, False))
+            self._all.contents[2] = (self._bottomline_focus, ('pack', None))
+        else:
+            self._all.contents[0] = (self._topline, ('pack', None))
+            inner.contents[0] = (self._vline, ('given', 1, False))
+            inner.contents[2] = (self._vline, ('given', 1, False))
+            self._all.contents[2] = (self._bottomline, ('pack', None))
+        return super().render(size, focus)
+
+
+class FocusLineBoxColor(urwid.WidgetDecoration, urwid.WidgetWrap):
+    def __init__(self, widget):
+        hline = urwid.Divider('─')
+        self._vline = urwid.AttrMap(urwid.SolidFill('│'), 'frame')
+        self._topline = urwid.AttrMap(
+            urwid.Columns([
+                ('fixed', 1, urwid.Text('┌')),
+                hline,
+                ('fixed', 1, urwid.Text('┐')),
+            ]),
+            'frame')
+        self._bottomline = urwid.AttrMap(
+            urwid.Columns([
+                ('fixed', 1, urwid.Text('└')),
+                hline,
+                ('fixed', 1, urwid.Text('┘')),
+            ]),
+            'frame')
+
+        self._middle = urwid.Columns(
+            [('fixed', 1, self._vline), widget, ('fixed', 1, self._vline)],
+            focus_column=1,
+        )
+        self._all = urwid.Pile(
+            [('flow', self._topline), self._middle, ('flow', self._bottomline)],
+            focus_item=1,
+        )
+
+        urwid.WidgetWrap.__init__(self, self._all)
+        urwid.WidgetDecoration.__init__(self, widget)
+
+    def render(self, size, focus):
+        if focus:
+            self._middle.contents[0][0].set_attr_map({None: 'frame focus color'})
+            self._all.contents[0][0].set_attr_map({None: 'frame focus color'})
+            self._all.contents[2][0].set_attr_map({None: 'frame focus color'})
+        else:
+            self._middle.contents[0][0].set_attr_map({None: 'frame'})
+            self._all.contents[0][0].set_attr_map({None: 'frame'})
+            self._all.contents[2][0].set_attr_map({None: 'frame'})
+        return super().render(size, focus)
+
+
+class FocusLineBoxTop(urwid.WidgetDecoration, urwid.WidgetWrap):
+    def __init__(self, widget):
+        topline = urwid.AttrMap(urwid.Divider('━'), 'frame')
+        self._all = urwid.Pile([('flow', topline), widget], focus_item=1)
+        urwid.WidgetWrap.__init__(self, self._all)
+        urwid.WidgetDecoration.__init__(self, widget)
+
+    def render(self, size, focus):
+        if focus:
+            self._all.contents[0][0].set_attr_map({None: 'frame focus top'})
+        else:
+            self._all.contents[0][0].set_attr_map({None: 'frame'})
+        return super().render(size, focus)
+
+
+linebox = {
+    'color': FocusLineBoxColor,
+    'top': FocusLineBoxTop,
+    'width': FocusLineBoxWidth,
+    'False': urwid.WidgetPlaceholder,
+}
