@@ -553,6 +553,7 @@ def test_color_option(runner):
 def choices(dateformat=0, timeformat=0,
             parse_vdirsyncer_conf=True,
             create_vdir=False,
+            default_calendar='',
             write_config=True):
     """helper function to generate input for testing `configure`"""
     confirm = {True: 'y', False: 'n'}
@@ -563,7 +564,9 @@ def choices(dateformat=0, timeformat=0,
     ]
     if not parse_vdirsyncer_conf:
         out.append(confirm[create_vdir])
+    out.append(default_calendar)
     out.append(confirm[write_config])
+    out.append('')
     return '\n'.join(out)
 
 
@@ -642,6 +645,9 @@ dateformat = %Y-%m-%d
 longdateformat = %Y-%m-%d
 datetimeformat = %Y-%m-%d %H:%M
 longdatetimeformat = %Y-%m-%d %H:%M
+
+[default]
+default_calendar = events_local
 '''
 
     # if aborting, no config file should be written
@@ -699,7 +705,7 @@ def test_printics_read_from_stdin(runner):
     runner = runner(command='printics')
     result = runner.invoke(main_khal, ['printics'], input=_get_text('cal_d'))
     assert not result.exception
-    assert '1 events found in stdin input\n An Event\n' in result.output
+    assert '1 events found in stdin input\n09.04.-09.04. An Event\n' in result.output
 
 
 def test_configure_command_config_exists(runner):
@@ -735,6 +741,9 @@ dateformat = %Y-%m-%d
 longdateformat = %Y-%m-%d
 datetimeformat = %Y-%m-%d %H:%M
 longdatetimeformat = %Y-%m-%d %H:%M
+
+[default]
+default_calendar = private
 '''.format(runner.xdg_data_home)
 
     # running configure again, should yield another vdir path, as the old
@@ -752,12 +761,25 @@ longdatetimeformat = %Y-%m-%d %H:%M
     assert '{}/khal/calendars/private1' .format(runner.xdg_data_home) in actual_config
 
 
+def cleanup(paths):
+    """reset permissions of all files and folders in `paths` to 644 resp. 755"""
+    for path in paths:
+        if os.path.exists(path):
+            os.chmod(str(path), 0o755)
+        for dirpath, dirnames, filenames in os.walk(path):
+            os.chmod(str(dirpath), 0o755)
+            for filename in filenames:
+                os.chmod(str(os.path.join(dirpath, filename)), 0o644)
+
+
 def test_configure_command_cannot_write_config_file(runner):
     runner = runner()
     runner.config_file.remove()
     os.chmod(str(runner.xdg_config_home), 555)
     result = runner.invoke(main_khal, ['configure'], input=choices())
     assert result.exit_code == 1
+    # make sure pytest can clean up behind us
+    cleanup([runner.xdg_config_home])
 
 
 def test_configure_command_cannot_create_vdir(runner):
@@ -770,6 +792,8 @@ def test_configure_command_cannot_create_vdir(runner):
     )
     assert 'Exiting' in result.output
     assert result.exit_code == 1
+    # make sure pytest can clean up behind us
+    cleanup([runner.xdg_data_home])
 
 
 def test_configure_no_vdir(runner):
