@@ -1,18 +1,18 @@
-import os.path
 import datetime as dt
-from validate import VdtValueError
+import os.path
 
 import pytest
+from khal.settings import get_config
+from khal.settings.exceptions import (CannotParseConfigFileError,
+                                      InvalidSettingsError)
+from khal.settings.utils import (config_checks, get_all_vdirs,
+                                 get_color_from_vdir, get_unique_name,
+                                 is_color)
 from tzlocal import get_localzone
+from validate import VdtValueError
 
 from .utils import LOCALE_BERLIN
 
-from khal.settings import get_config
-from khal.settings.exceptions import InvalidSettingsError, \
-    CannotParseConfigFileError
-from khal.settings.utils import get_all_vdirs, get_unique_name, config_checks, \
-    get_color_from_vdir, is_color
-
 PATH = __file__.rsplit('/', 1)[0] + '/configs/'
 
 
@@ -26,19 +26,20 @@ class TestSettings(object):
         comp_config = {
             'calendars': {
                 'home': {'path': os.path.expanduser('~/.calendars/home/'),
-                         'readonly': False, 'color': None, 'type': 'calendar'},
+                         'readonly': False, 'color': None, 'priority': 10, 'type': 'calendar'},
                 'work': {'path': os.path.expanduser('~/.calendars/work/'),
-                         'readonly': False, 'color': None, 'type': 'calendar'},
+                         'readonly': False, 'color': None, 'priority': 10, 'type': 'calendar'},
             },
             'sqlite': {'path': os.path.expanduser('~/.local/share/khal/khal.db')},
             'locale': LOCALE_BERLIN,
             'default': {
-                'default_command': 'calendar',
                 'default_calendar': None,
                 'print_new': 'False',
                 'highlight_event_days': False,
                 'timedelta': dt.timedelta(days=2),
-                'show_all_days': False
+                'default_event_duration': dt.timedelta(days=1),
+                'default_dayevent_duration': dt.timedelta(hours=1),
+                'show_all_days': False,
             }
         }
         for key in comp_config:
@@ -48,6 +49,10 @@ class TestSettings(object):
         with pytest.raises(InvalidSettingsError):
             get_config(PATH + 'nocalendars.conf')
 
+    def test_one_level_calendar(self):
+        with pytest.raises(InvalidSettingsError):
+            get_config(PATH + 'one_level_calendars.conf')
+
     def test_small(self):
         config = get_config(
             PATH + 'small.conf',
@@ -57,30 +62,32 @@ class TestSettings(object):
         comp_config = {
             'calendars': {
                 'home': {'path': os.path.expanduser('~/.calendars/home/'),
-                         'color': 'dark green', 'readonly': False,
+                         'color': 'dark green', 'readonly': False, 'priority': 20,
                          'type': 'calendar'},
                 'work': {'path': os.path.expanduser('~/.calendars/work/'),
-                         'readonly': True, 'color': None,
+                         'readonly': True, 'color': None, 'priority': 10,
                          'type': 'calendar'}},
             'sqlite': {'path': os.path.expanduser('~/.local/share/khal/khal.db')},
             'locale': {
                 'local_timezone': get_localzone(),
                 'default_timezone': get_localzone(),
-                'timeformat': '%H:%M',
-                'dateformat': '%d.%m.',
-                'longdateformat': '%d.%m.%Y',
-                'datetimeformat': '%d.%m. %H:%M',
-                'longdatetimeformat': '%d.%m.%Y %H:%M',
+                'timeformat': '%X',
+                'dateformat': '%x',
+                'longdateformat': '%x',
+                'datetimeformat': '%c',
+                'longdatetimeformat': '%c',
                 'firstweekday': 0,
                 'unicode_symbols': True,
                 'weeknumbers': False,
             },
             'default': {
                 'default_calendar': None,
-                'default_command': 'calendar',
                 'print_new': 'False',
                 'highlight_event_days': False,
                 'timedelta': dt.timedelta(days=2),
+                'default_event_duration': dt.timedelta(days=1),
+                'default_dayevent_duration': dt.timedelta(hours=1),
+
                 'show_all_days': False
             }
         }
@@ -99,7 +106,6 @@ timeformat: %H:%M
 dateformat: %d.%m.
 longdateformat: %d.%m.%Y
 [default]
-default_command: calendar
 """
         conf_path = str(tmpdir.join('old.conf'))
         with open(conf_path, 'w+') as conf:
@@ -224,60 +230,70 @@ def test_config_checks(metavdirs):
                 'path': '/cal3/home',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
             'my calendar': {
                 'color': 'dark blue',
                 'path': '/cal1/public',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
             'my private calendar': {
                 'color': '#FF00FF',
                 'path': '/cal1/private',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
             'public': {
                 'color': None,
                 'path': '/cal2/public',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
             'public1': {
                 'color': None,
                 'path': '/cal3/public',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
             'work': {
                 'color': None,
                 'path': '/cal3/work',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
             'cfgcolor': {
                 'color': 'dark blue',
                 'path': '/cal4/cfgcolor',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
             'dircolor': {
                 'color': 'dark blue',
                 'path': '/cal4/dircolor',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
             'cfgcolor_again': {
                 'color': 'dark blue',
                 'path': '/cal4/cfgcolor_again',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
             'cfgcolor_once_more': {
                 'color': 'dark blue',
                 'path': '/cal4/cfgcolor_once_more',
                 'readonly': False,
                 'type': 'calendar',
+                'priority': 10,
             },
 
         },
